# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from . import medleydb_melody
from . import medleydb_pitch
from . import orchset
from . import ikala
from . import beatles
from . import salami
